// -*- C++ -*-
#ifndef Herwig_RPVFFZVertex_H
#define Herwig_RPVFFZVertex_H
//
// This is the declaration of the RPVFFZVertex class.
//

#include "ThePEG/Helicity/Vertex/Vector/FFVVertex.h"
#include "Herwig/Models/Susy/SusyBase.h"
#include "Herwig/Models/Susy/MixingMatrix.fh"

namespace Herwig {

using namespace ThePEG;

/**
 * Here is the documentation of the RPVFFZVertex class.
 *
 * @see \ref RPVFFZVertexInterfaces "The interfaces"
 * defined for RPVFFZVertex.
 */
class RPVFFZVertex: public Helicity::FFVVertex {

public:

  /**
   * The default constructor.
   */
  RPVFFZVertex();

  /**
   * Calculate the couplings.
   * @param q2 The scale \f$q^2\f$ for the coupling at the vertex.
   * @param part1 The ParticleData pointer for the first  particle.
   * @param part2 The ParticleData pointer for the second particle.
   * @param part3 The ParticleData pointer for the third  particle.
   */
  virtual void setCoupling(Energy2 q2, tcPDPtr part1,
                           tcPDPtr part2, tcPDPtr part3);

  /** @name Functions used by the persistent I/O system. */
  //@{
  /**
   * Function used to write out object persistently.
   * @param os the persistent output stream written to.
   */
  void persistentOutput(PersistentOStream & os) const;

  /**
   * Function used to read in object persistently.
   * @param is the persistent input stream read from.
   * @param version the version number of the object when written.
   */
  void persistentInput(PersistentIStream & is, int version);
  //@}

  /**
   * The standard Init function used to initialize the interfaces.
   * Called exactly once for each class by the class description system
   * before the main function starts or
   * when this class is dynamically loaded.
   */
  static void Init();

protected:

  /** @name Clone Methods. */
  //@{
  /**
   * Make a simple clone of this object.
   * @return a pointer to the new object.
   */
  virtual IBPtr clone() const;

  /** Make a clone of this object, possibly modifying the cloned object
   * to make it sane.
   * @return a pointer to the new object.
   */
  virtual IBPtr fullclone() const;
  //@}

protected:

  /** @name Standard Interfaced functions. */
  //@{
  /**
   * Initialize this object after the setup phase before saving an
   * EventGenerator to disk.
   * @throws InitException if object could not be initialized properly.
   */
  virtual void doinit();
  //@}

private:

  /**
   * The assignment operator is private and must never be called.
   * In fact, it should not even be implemented.
   */
  RPVFFZVertex & operator=(const RPVFFZVertex &) = delete;

private:

  /**
   * The value of \f$sin(\theta_W)\f$
   */
  double _sw;
  
  /**
   * The value of \f$cos(\theta_W)\f$
   */
  double _cw;

  /**
   * Store the neutralino mixing matrix
   */
  tMixingMatrixPtr _theN;

  /**
   * The U mixing matrix
   */
  tMixingMatrixPtr _theU;

  /**
   * The V mixing matrix
   */
  tMixingMatrixPtr _theV;

  /**
   * Store the id of the 1st neutralino when the coupling was last evaluated
   */
  long _id1last;

  /**
   * Store the id of the 2nd neutralino when the coupling was last evaluated
   */
  long _id2last;

  /**
   * Store the value at which the coupling when it was last evalutated
   */
  Energy2 _q2last;

  /**
   * Store the value of the coupling when it was last evalutated
   */
  Complex _couplast;

  /**
   * Store the value of the left-coupling when it was last evalutated
   */
  Complex _leftlast;

  /**
   * Store the value of the right-coupling when it was last evalutated
   */
  Complex _rightlast;

  /**
   *  The left couplings of the Standard Model fermions.
   */
  vector<double> _gl;

  /**
   *  The right couplings of the Standard Model fermions.
   */
  vector<double> _gr;

  /**
   * The ID of the gauge boson when the vertex was last evaluated
   */
  long _gblast;

  /**
   *  Switch controlling which subset of interactions to include
   */
  unsigned int _interactions;

};

}

#endif /* Herwig_RPVFFZVertex_H */
